/***************************************************************************
*    Client Server implementation with sample application front end.	   *
*									   *
*    Copyright (C) 1994 Akshay Srinivasan				   *
*									   *
*    This program is free software; you can redistribute it and/or modify  *
*    it under the terms of the GNU General Public License as published by  *
*    the Free Software Foundation; either version 2 of the License, or	   *
*    (at your option) any later version.				   *
*									   *
*    This program is distributed in the hope that it will be useful,	   *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of	   *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	   *
*    GNU General Public License for more details.			   *
*									   *
*    You should have received a copy of the GNU General Public License	   *
*    along with this program; if not, write to the Free Software	   *
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		   *
*									   *
*    The author can be reached via e-mail to srin9340@nova.gmi.edu.	   *
***************************************************************************/

/***************************************************************************
*									   *
*	CLIENT.C by srin9340@nova.gmi.edu 1994				   *
*	An implementation of a multiplexed client server using poll.	   *
*									   *
***************************************************************************/
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <poll.h>
#include <stdio.h>
#include <string.h>
#include <curses.h>
#include "config.h"

#define MAX_ROWS	25
#define MAX_COLS	70
#define ESC		033
#define CR		015
#define BS		010
#define DEL		0177

/* Global variables that are the equivalent of one of the client structs */
/* in the server program.						 */
char buf[1024];
int reads=0, sd=0;

/*************************************************************************
*  Gets the name of the server machine location, the port number and the *
*  string you want to broadcast to the other clients.			 *
*************************************************************************/
main(argc, argv)
int argc;
char *argv[];
{
	int count=0;
	struct sockaddr_in server;
	struct hostent *hp, *gethostbyname();

	/* Initialize curses. */
	init_curses();

	/* Reports incorrect usage */
	if (argc != 2) {
		perror("Usage:: client (name)");
		exit(1);
		}

	/* Create a socket */
	if ((sd=socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("Creating connect socket.");
		exit(1);
		}

	/* Initialize the server structure */
	server.sin_family = AF_INET;
	/* Converts the machine name from the command line to an IP number. */
	if ((hp = gethostbyname(SERVER_HOST_NAME)) == 0) {
		perror("Host specified is unknown.");
		exit(1);
		}
	bcopy((char *)hp->h_addr, (char *)&server.sin_addr, hp->h_length);
	server.sin_port = SERVER_PORT_NUMBER;

	/* Try to connect to the server */
	if (connect(sd, (struct sockaddr *)&server, sizeof(server)) < 0) {
		perror("Connecting to server socket.");
		exit(1);
		}

	move(27,0);
	addstr("----BBCLI----Bombay Broadcast CLIent----BBCLI----Bombay Broadcast CLIent----");

	/* Continuously try to read and write to the socket. */
	while(1) {
		readserv();
		if (reads) {
			dispmsgs();
			reads=0;
			}
		sendmsg(argv[1]);
		}
}



/**************************************************
*  If the socket is ready then read from it.	  *
*  Also set the read bit so it will be displayed. *
**************************************************/
readserv()
{
	struct pollfd cpfd;
	int r;

	/* Initialize poll structure */
	cpfd.fd = sd;
	cpfd.events = POLLIN;
	/* If there is a message from the server read it. */
	if (poll(&cpfd, 1, 0) > 0) {
		if (cpfd.revents & POLLIN) {
			r = read(sd, buf, 1024);
			if (r==0) {
				close(sd);
				perror("Lost the server.");
				exit(1);
			} else if (r>0) {
				reads=1;
				}
			}
		}
}


/***************************************************
*  Write the message to the socket if it is ready. *
***************************************************/
writeserv()
{
	struct pollfd cpfd;

	cpfd.fd = sd; 
	cpfd.events = POLLOUT;
	if (poll(&cpfd, 1, 0) > 0) {
		if (cpfd.revents & POLLOUT) {
			if (write(sd, buf, sizeof(buf)) < 0) {
				perror("Writing server socket");
				}
			}
		}
}


/***************************************************
*  Handling messages from server.
***************************************************/
dispmsgs()
{
	static int row=1, col=1;
	int count=0;

	while(buf[count]!='\0') {
		if (++col==MAX_COLS) {
			col=1;
			if (++row==MAX_ROWS)
				row=1;
			clrtoeol();
			}
		move(row,col);
		addch(buf[count++]);
		}

	col=1;
	if (++row==MAX_ROWS)
		row=1;
	move(row,col);
	clrtoeol();

	refresh();
}


/***************************************************
*  Initializing terminal for curses.
***************************************************/
init_curses()
{
	initscr();
	cbreak();
	nonl();
	noecho();
	keypad(stdscr,TRUE);
	nodelay(stdscr,TRUE);
}


/***************************************************
*  The users input is got and sent off to server.
***************************************************/
sendmsg(argv)
char *argv;
{
	static int row=29, col=1, count=0;
	int newch=0;
	static char tmp[950];

	if ((newch=getch())==ERR)
		return;
	switch(newch) {
		case(KEY_LEFT):
		case(KEY_RIGHT):
		case(KEY_UP):
		case(KEY_DOWN):
			break;
		case(ESC):
			endwin();
			exit(0);
			break;
		case(CR):
			tmp[count]='\0';
			sprintf(buf,"(%s) %s",argv, tmp);
			writeserv();
			col=1;
			row=29;
			move(row,col);
			clrtobot();
			count=0;
			refresh();
			return;
			break;
		case(BS):
		case(DEL):
			move(row,col);
			delch();
			if (col>0) {
				col--;
			} else {
				flash();
				break;
				}
			count--;
			break;
		default:
			if (count<900) {
				tmp[count++]=newch;
			} else {
				tmp[count]='\0';
				sprintf(buf,"(%s) %s",argv, tmp);
				writeserv();
				count=0;
				refresh();
				return;
				}
			if (++col==MAX_COLS) {
				col=1;
				if (++row==35)
					row=29;
				move(row,col);
				clrtoeol();
				}
			move(row,col);
			addch(newch);
		}
	refresh();
}

	
	
