/***************************************************************************
*    Client Server implementation with sample application front end.	   *
*									   *
*    Copyright (C) 1994 Akshay Srinivasan				   *
*									   *
*    This program is free software; you can redistribute it and/or modify  *
*    it under the terms of the GNU General Public License as published by  *
*    the Free Software Foundation; either version 2 of the License, or	   *
*    (at your option) any later version.				   *
*									   *
*    This program is distributed in the hope that it will be useful,	   *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of	   *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	   *
*    GNU General Public License for more details.			   *
*									   *
*    You should have received a copy of the GNU General Public License	   *
*    along with this program; if not, write to the Free Software	   *
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		   *
*									   *
*    The author can be reached via e-mail to srin9340@nova.gmi.edu.	   *
***************************************************************************/

/***************************************************************************
*									   *
*	SERVER.C by srin9340@nova.gmi.edu 1994				   *
*	An implementation of a multiplexed client server using poll.	   *
*									   *
***************************************************************************/
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <poll.h>
#include <stdio.h>
#include "config.h"

#define MAX_USERS	2000

/*********************************
*  This is the client structure. *
*********************************/
struct conn {
	int sd;		/* socket descriptor */
	int active;	/* connection status */
	int read;	/* info has been read */
	char buf[1024];	/* info that has been read */
	};

struct conn users[MAX_USERS];	/* initialize array of structures for clients */


/*****************************************************************************
*  Zeros out the structures, initialize listening for new connections, loop  *
*  through accept, read and write functions.				     *
*****************************************************************************/
main()
{
	zeroUsers();

	init_listen();

	while(1) {
		pfnc();
		rfnc();
		}
}



/*******************************************
*  Zero out the uninitialized structures.  *
*******************************************/
zeroUsers()
{
	int i;

	for (i=0; i<MAX_USERS; i++) {
		users[i].sd=0;
		users[i].active=0;
		users[i].read=0;
		}
}



/***********************************************
*  Initialize the listen descriptor on which   *
*  the server accepts new connections.	       *
***********************************************/
init_listen()
{
	int length;
	struct sockaddr_in server;

	/* Create socket */
	if ((users[0].sd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("Opening listen socket");
		exit(1);
		}

	/* Create name with wildcards */
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = INADDR_ANY;
	server.sin_port = 43210;
	if (bind(users[0].sd, (struct sockaddr *)&server, sizeof(server)) < 0) {
		perror("Binding listen socket.");
		exit(1);
		}

	/* Find the assigned port and print it out. */
	length = sizeof(server);
	if (getsockname(users[0].sd, (struct sockaddr *)&server, &length) < 0) {
		perror("Could not get port number.");
		exit(1);
		}

	/*printf("Socket port #%d.\n", ntohs(server.sin_port));*/

	/* Start accepting connections. */
	listen(users[0].sd, 5);
}
	



/************************************************
*  Polls for new connections and messages from  *
*  active clients.			        *
************************************************/
pfnc()
{
	int c, r;
	struct pollfd tpfd;

	/* Initialize the poll structure */
	tpfd.fd = users[0].sd;
	tpfd.events = POLLIN;
	/* Poll the descriptor */
	if (poll(&tpfd, 1, 0) > 0) {
		/* If you have a new connection then find a free struct	*/
		/* and assign the new client to that struct.		*/
		if (tpfd.revents & POLLIN) {
			for (c=1; c<MAX_USERS; c++) {
				if (users[c].active==0) {
					if ((users[c].sd = accept(users[0].sd, (struct sockaddr *)0, (int *)0)) > 0) {
						users[c].active = 1;
						break;
						}
					}
				}
			}
		}

	/* Loop through all active descriptors for something to read.  If  */
	/* something needs to be read then set the read bit of that struct */
	/* and read the info into the structs buffer.			   */
	for (c=1; c<MAX_USERS; c++) {
		if (users[c].active == 1) {
			tpfd.fd = users[c].sd;
			tpfd.events = POLLIN;
			if (poll(&tpfd, 1, 0) > 0) {
				if (tpfd.revents & POLLIN) {
					r = read(users[c].sd, users[c].buf, 1024);
					if (r == 0) {
						close(users[c].sd);
						users[c].active=0;
					} else if (r > 0) {
						users[c].read = 1;
						}
					}
				}
			}
		}
}



/*****************************************************************************
*  Poll all active client structures for read status.  If something was read *
*  display it and also broadcast it to all clients.			     *
*****************************************************************************/
rfnc()
{
	int c,i;

	for (c=1; c<MAX_USERS; c++) {
		if (users[c].active & users[c].read) {
			/*printf("%s\n",users[c].buf);*/
			users[c].read = 0;
			for (i=1; i<MAX_USERS; i++)
				if (users[i].active)
					write(users[i].sd, users[c].buf, sizeof(users[c].buf));
			}
		}
}
